#include "config.h"
#include <OctoPrintAPI.h>
#include <WiFiClient.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include "tunes.h"

void playTune(int tuneNumber);
WiFiClient client;

OctoprintApi api(client, OCTOPRINT_HOST, OCTOPRINT_PORT, OCTOPRINT_API_KEY);
unsigned long api_mtbs = OCTOPRINT_API_REFRESH_INTERVAL;
unsigned long api_lasttime = 0;

bool displayOctoPrintVersion(Adafruit_SSD1306& display) {
    display.clearDisplay();
    display.setCursor(0, 0);
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.print("Fetching OctoPrint Version...");
    display.display();

    if (api.getOctoprintVersion()) {
        display.clearDisplay();
        display.setCursor(0, 0);
        display.setTextSize(1);
        display.setTextColor(WHITE);
        display.print("OctoPrint Version:");
        display.setCursor(0, 10);
        display.setTextSize(2);
        display.print(api.octoprintVer.octoprintServer);
        display.display();
        playTune(2);
        return true;
    } else {
        display.clearDisplay();
        display.setCursor(0, 0);
        display.setTextSize(1);
        display.setTextColor(WHITE);
        display.print("Failed to get");
        display.setCursor(0, 10);
        display.print("OctoPrint version.");
        display.display();
        playTune(3);
        return false;
    }
    delay(1000);
}

bool fetchPrintingStatus(Adafruit_SSD1306& display) {
    if (millis() - api_lasttime > api_mtbs || api_lasttime == 0 && !is_display_off) {
        if (api.getPrinterStatistics()) {
            display.clearDisplay();
            display.setCursor(0, 0);
            display.setTextSize(1);
            display.setTextColor(WHITE);
            // Display printer state
            display.println(api.printerStats.printerState);
            display.drawLine(0,10,127,10,WHITE);
            display.drawLine(63,10,63,22,WHITE);
            // Display nozzle and bed temperature
            display.drawBitmap(10, 13, bitmap_nozzle, 8, 8, WHITE);
            display.setCursor(21, 13);
            display.print(api.printerStats.printerTool0TempActual);
            display.drawBitmap(74, 13, bitmap_bed, 8, 8, WHITE);
            display.setCursor(85, 13);
            display.print(api.printerStats.printerBedTempActual);

            //Estimated print time if availble in human readable time HH:MM:SS
            int estrunHours= api.printJob.estimatedPrintTime/3600;
            int estsecsRemaining=api.printJob.estimatedPrintTime%3600;
            int estrunMinutes=estsecsRemaining/60;
            int estrunSeconds=estsecsRemaining%60;
            char estbuf[31];
            sprintf(estbuf,"%02d:%02d:%02d",estrunHours,estrunMinutes,estrunSeconds);
            display.setCursor(80, 0);
            display.println(estbuf);

            // const float temp_percent = floor(api.printJob.progressCompletion*100)/100;
            // display.print(temp_percent);
            // display.println("%");
            display.display();
            api_lasttime = millis(); // Update the last fetch time
            return true;
        } else {
            Serial.println("Failed to fetch printer statistics.");
            return false;
        }
    }
    return false; // No fetch was done (within the time interval)
}
